<?php

/**
* @file
* Contains \Drupal\link_icons\Plugin\Field\FieldFormatter\LinkIconsFormatter.
*/

namespace Drupal\link_icons\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\Core\Render\Markup;

/**
* Plugin implementation of the 'link_icons_formatter' formatter.
*
* @FieldFormatter (
*   id = "link_icons_formatter",
*   label = @Translation("Link with service icon"),
*   field_types = {
*     "link"
*   }
* )
*/
class LinkIconsFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form['text'] = array(
      '#type' => 'select',
      '#title' => $this->t('Text alongside the icon'),
      '#description' => $this->t('Select what text should appear with each icon (if any)'),
      '#default_value' => $this->getSetting('text'),
      '#required' => FALSE,
      '#empty_value' => 'none',
      '#options' => array(
        'title' => $this->t('Link title'),
        'URL' => $this->t('Link URL'),
        'title - URL' => $this->t('Link title - link URL'),
        'title: URL' => $this->t('Link title: link URL'),
        'URL (title)' => $this->t('link URL (Link title)'),
      ),
    );

    $form['order'] = array(
      '#type' => 'select',
      '#title' => $this->t('Order of icon and text'),
      '#description' => $this->t('Select the order of each icon and its text (if any)'),
      '#default_value' => $this->getSetting('order'),
      '#required' => TRUE,
      '#options' => array(
        'first' => $this->t('Icon first'),
        'last' => $this->t('Icon last'),
      ),
    );

    $form['size'] = array(
      '#type' => 'select',
      '#title' => $this->t('Size of the icon'),
      '#description' => $this->t('Select what size each icon should be (not used/supported if icons have a background [see setting below])'),
      '#default_value' => $this->getSetting('size'),
      '#required' => TRUE,
      '#options' => array(
        '1x' => $this->t('100%'),
        'lg' => $this->t('133%'),
        '2x' => $this->t('200%'),
        '3x' => $this->t('300%'),
        '4x' => $this->t('400%'),
        '5x' => $this->t('500%'),
      ),
    );

    $form['width'] = array(
      '#type' => 'select',
      '#title' => $this->t('Width of icons'),
      '#description' => $this->t('Select whether each icon should have a fixed or variable width (helps with horizontal alignment)'),
      '#default_value' => $this->getSetting('width'),
      '#required' => TRUE,
      '#options' => array(
        'fixed' => $this->t('Fixed width'),
        'variable' => $this->t('Variable width'),
      ),
    );

    $form['coloured'] = array(
      '#type' => 'select',
      '#title' => $this->t('Coloured icons'),
      '#description' => $this->t("Select whether each service icon should be coloured correctly (if that's possible with a single colour)"),
      '#default_value' => $this->getSetting('coloured'),
      '#required' => TRUE,
      '#options' => array(
        'coloured' => $this->t('Coloured'),
        'uncoloured' => $this->t('Uncoloured'),
      ),
    );

    $form['shaped'] = array(
      '#type' => 'select',
      '#title' => $this->t('Pre-shaped icons'),
      '#description' => $this->t('Select whether each service icon should be the pre-squared or pre-circled version (if available)'),
      '#default_value' => $this->getSetting('shaped'),
      '#required' => TRUE,
      '#options' => array(
        'squared' => $this->t('Squared'),
        'circled' => $this->t('Circled'),
        'squared or else circled' => $this->t('Squared, or else circled'),
        'circled or else squared' => $this->t('Circled, or else squared'),
        'natural' => $this->t('Natural'),
      ),
    );

    $form['background'] = array(
      '#type' => 'select',
      '#title' => $this->t('Background of the icons'),
      '#description' => $this->t('Select what background each icon should have (if any)'),
      '#default_value' => $this->getSetting('background'),
      '#required' => FALSE,
      '#empty_value' => 'none',
      '#options' => array(
        'square' => $this->t('Square with rounded corners solid'),
        'square-o' => $this->t('Square with rounded corners outline'),
        'stop' => $this->t('Square solid'),
        'play' => $this->t('Triangle pointing right (play symbol) solid'),
        'circle'  => $this->t('Circle solid'),
        'circle-o' => $this->t('Circle outline'),
        'circle-o-notch' => $this->t('Circle outline with notch at the top'),
        'circle-thin' => $this->t('Circle outline hairline'),
        'dot-circle-o' => $this->t('Circle outline with centre dot'),
        'cloud' => $this->t('Cloud solid'),
        'sun-o' => $this->t('Sun outline'),
        'folder' => $this->t('Folder solid'),
        'folder-o' => $this->t('Folder outline'),
        'file' => $this->t('File solid'),
        'file-o' => $this->t('File outline'),
        'heart' => $this->t('Heart solid'),
        'heart-o' => $this->t('Heart outline'),
        'laptop' => $this->t('Laptop'),
        'tablet' => $this->t('Tablet'),
      ),
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = array();

    $summary[] = t('Output @text text with @size size @width width @coloured @shaped icons, @background backgrounds and the icons placed @order.',
      array(
        '@text' => ($this->getSetting('text') == 'none' ? 'no' : '"' . $this->getSetting('text') . '"'),
        '@order' => $this->getSetting('order'),
        '@size' => $this->getSetting('size'),
        '@width' => $this->getSetting('width'),
        '@coloured' => $this->getSetting('coloured'),
        '@shaped' => $this->getSetting('shaped'),
        '@background' => ($this->getSetting('background') == 'none' ? 'no' : 'fa-' . $this->getSetting('background')),
      )
    );
    return $summary;
  }


  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return array(
      'text' => 'title',
      'order' => 'first',
      'size' => '1x',
      'width' => 'fixed',
      'coloured' => 'coloured',
      'shaped' => 'natural',
      'background' => 'none',
    ) + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = array();

    $services = array(
      // Format is:
      // Hostname => array(ID, main fa icon name, colour, square fa icon name, circle fa icon name).
      '500px.com' => array('fivehundredpx', '500px', 'black'),
      'amazon.com' => array('amazon', 'amazon', 'black'),
      'amazon.co.uk' => array('amazon', 'amazon', 'black'),
      'angel.co' => array('angel', 'angellist', 'black'),
      'apple.com' => array('apple', 'apple', '#6E6E6E'),
      'bandcamp.com' => array('bandcamp', 'bandcamp', '#639AA9'),
      'behance.net' => array('behance', 'behance', 'black', 'behance-square'),
      'bitbucket.org' => array('bitbucket', 'bitbucket', 'black', 'bitbucket-square'),
      'chrome.google.com' => array('chrome', 'chrome', '#E04B40'),
      'codepen.io' => array('codepen', 'codepen', 'black'),
      'del.icio.us' => array('delicious', 'delicious', 'black'),
      'deviantart.com' => array('deviantart', 'deviantart', '#CFDD20'),
      'digg.com' => array('digg', 'digg', 'black'),
      'dribbble.com' => array('dribbble', 'dribbble', '#CA2A68'),
      'dropbox.com' => array('dropbox', 'dropbox', '#1182DE'),
      'drupal.org' => array('drupal', 'drupal', '#0073BA'),
      'eercast.com' => array('eercast', 'eercast', '#F1592A'),
      'etsy.com' => array('etsy', 'etsy', '#F56400'),
      'facebook.com' => array('facebook', 'facebook', '#4965A0', 'facebook-square'),
      'flickr.com' => array('flickr', 'flickr'),
      'fontawesome.com' => array('fontawesome', 'font-awesome', '#228AE6'),
      'fontawesome.io' => array('fontawesome', 'font-awesome', '#1D9E74'),
      'foursquare.com' => array('foursquare', 'foursquare', '#38AAE1'),
      'freecodecamp.com' => array('freecodecamp', 'freecodecamp', '#006400'),
      'getgrav.com' => array('grav', 'grav', 'black'),
      'github.com' => array('github', 'github', 'black', 'github-square'),
      'google.com' => array('google', 'google'),
      'imdb.com' => array('imdb', 'imdb', '#E4BB36'),
      'instagram.com' => array('instagram', 'instagram', '#48769B'),
      'joomla.org' => array('joomla', 'joomla', '#1B63A2'),
      'jsfiddle.net' => array('jsfiddle', 'jsfiddle', '#4679BD'),
      'last.fm' => array('last_fm', 'lastfm', '#B90000', 'lastfm-square'),
      'linkedin.com' => array('linkedin', 'linkedin', '#1C87BD', 'linkedin-square'),
      'linode.com' => array('linode', 'linode', '#00B259'),
      'medium.com' => array('medium', 'medium', '#00AB6B'),
      'meetup.com' => array('meetup', 'meetup', '#ED1C40'),
      'paypal.com' => array('paypal', 'paypal', '#18130D'),
      'paypal.me' => array('paypal', 'paypal', '#18130D'),
      'pinterest.com' => array('pinterest', 'pinterest', '#DC1E26', 'pinterest-square'),
      'play.google.com' => array('google_play', 'android', '#689F38'),
      'plus.google.com' => array('google_plus', 'google-plus', '#DD4B39', 'google-plus-square', 'google-plus-circle'),
      'producthunt.com' => array('product_hunt', 'product-hunt', '#DA552F'),
      'quora.com' => array('quora', 'quora', '#B92B27'),
      'ravelry.com' => array('ravelry', 'ravelry', '#C5154F'),
      'reddit.com' => array('reddit', 'reddit', 'black', 'reddit-square'),
      'slideshare.net' => array('slideshare', 'slideshare', 'black'),
      'snapchat.com' => array('snapchat', 'snapchat', '#FFFC01', 'snapchat-square'),
      'soundcloud.com' => array('soundcloud', 'soundcloud', '#FE4E00'),
      'spotify.com' => array('spotify', 'spotify', '#8EBD05'),
      'stackexchange.com' => array('stack_exchange', 'stack-exchange', '#195398'),
      'stackoverflow.com' => array('stack_overflow', 'stack-overflow', '#FE7A15'),
      'steamcommunity.com' => array('steam', 'steam', 'black', 'steam-square'),
      'steampowered.com' => array('steam', 'steam', 'black', 'steam-square'),
      'stumbleupon.com' => array('stumbleupon', 'stumbleupon', '#EA4B24', '', 'stumbleupon-circle'),
      'telegram.org' => array('telegram', 'telegram', '#179CDE'),
      'tripadvisor.com' => array('tripadvisor', 'tripadvisor', 'black'),
      'tripadvisor.co.uk' => array('tripadvisor', 'tripadvisor', 'black'),
      'tumblr.com' => array('tumblr', 'tumblr', '#2C4762', 'tumblr-square'),
      'twitch.tv' => array('twitch', 'twitch', '#4B367C'),
      'twitter.com' => array('twitter', 'twitter', '#00ACEE', 'twitter-square'),
      'viadeo.com' => array('viadeo', 'viadeo', '#EA6725', 'viadeo-square'),
      'vimeo.com' => array('vimeo', 'vimeo-square', '#1AB7EA'),
      'vine.co' => array('vine', 'vine', '#00BF8F'),
      'wikipedia.org' => array('wikipedia', 'wikipedia-w', 'black'),
      'wordpress.com' => array('wordpress', 'wordpress', '#0074A2'),
      'wordpress.org' => array('wordpress', 'wordpress', '#0074A2'),
      'wpexplorer.com' => array('wpexplorer', 'wpexplorer', '#1796C6'),
      'xing.com' => array('xing', 'xing', '#006464', 'xing-square'),
      'yahoo.com' => array('yahoo', 'yahoo', '#2D1152'),
      'yelp.com' => array('yelp', 'yelp', '#D32323'),
      'yelp.co.uk' => array('yelp', 'yelp', '#D32323'),
      'youtube.com' => array('youtube', 'youtube', '#CC181E', 'youtube-square'),
      // TODO: Add more icons/services from Font Awesome.
    );

    // Step through the items
    foreach ($items as $delta => $item) {
      $parsed = parse_url($item->getUrl()->toString());
      $chunks = explode('.', $parsed['host']);
      $host3 = '';
      if (count($chunks) == 1) {
        $host = $chunks[0];
      } else {
        $host = $chunks[count($chunks) - 2] . '.' . $chunks[count($chunks) - 1];
        if (count($chunks) > 2) {
          $host3 = $chunks[count($chunks) - 3] . '.' . $host;
        }
      }

      // Find the service from $services, or else make it a generic navy globe 
      if (isset($host3) && isset($services[$host3])) {
        $service = $services[$host3];
      } else if (isset($services[$host])) {
        $service = $services[$host];
      } else { 
        $service = array('generic', 'globe', 'navy');
      }

      $hollow_endings = array(
        '-o',
        '-o-notch',
        '-thin',
        'sun',
        'laptop',
        'tablet',
      );

      // Determine if the icon needs to be inverted
      if ($this->getSetting('background') != 'none') {
        $inverse = TRUE;
        foreach ($hollow_endings as $ending) {
          if ($inverse) {
            $ending_length = strlen($ending);
            if ($ending_length <= strlen($this->getSetting('background'))) {
              if (substr_compare($this->getSetting('background'), $ending, -$ending_length) === 0) {
                $inverse = FALSE;
              }
            }
          }
        }
      }

      switch($this->getSetting('shaped')) {
        case 'squared':
          $icon = (isset($service[3]) ? $service[3] : $service[1]);
          break;

        case 'squared or else circled':
          $icon = (isset($service[3]) ? $service[3] : (isset($service[4]) ? $service[4] : $service[1]));
          break;

        case 'circled':
          $icon = (isset($service[4]) ? $service[4] : $service[1]);
          break;

        case 'circled or else squared':
          $icon = (isset($service[4]) ? $service[4] : ((isset($service[3]) && !is_null($service[3])) ? $service[3] : $service[1]));
          break;

        case 'natural':
        default:
          $icon = $service[1];
          break;
      }

      // Generate the icon link HTML
      $icon_link = Link::fromTextAndUrl(
        Markup::create(
          ($this->getSetting('background') != 'none' ? '<span class="fa-stack'
              . ($this->getSetting('size') != '1x' ? ' fa-' . $this->getSetting('size') : '')
              . '"><i class="fa fa-'
              . $this->getSetting('background')
              . ' fa-stack-2x"></i>'
            : ''
          )
          . '<i class="service ' . $service[0] . ' fa' . ($service[0] != 'generic' ? 'b' : 's') . ' fa-' . $icon
          . ($this->getSetting('background') == 'none' && $this->getSetting('size') != '1x' ? ' fa-' . $this->getSetting('size') : '')
          . ($this->getSetting('width') == 'fixed' ? ' fa-fw' : '')
          . ($this->getSetting('background') != 'none' ? ' fa-stack-1x' . (isset($inverse) && $inverse ? ' fa-inverse' : '') : '')
          . '"'
          . ($this->getSetting('coloured') != 'uncoloured' ? ' style="color: ' . $service[2] . ';"' : '')
          . '></i>'
          . ($this->getSetting('background') != 'none' ? '</span>' : '')
        ),
        Url::fromUri($item->getUrl()->toString(),
          array(
            'attributes' => array(
              'title' => $item->title,
              'target' => '_blank',
            ),
          )
        )         
      )->toString();

      // Generate the text HTML, if required, and combine it with the link icon HTML in $markup
      $text = '';
      if ($this->getSetting('text') != 'none') {
        switch ($this->getSetting('text')) {
          case 'title':
            $text = $item->title;
            break;

          case 'URL':
            $text = $item->getUrl()->toString();
            break;

          case 'title - URL':
            $text = $item->title . ' - ' . $item->getUrl()->toString();
            break;

          case 'title: URL':
            $text = $item->title . ': ' . $item->getUrl()->toString();
            break;

          case 'URL (title)':
            $text = $item->getUrl()->toString() . ' (' . $item->title . ')';
            break;
        }

        $text_link = Link::fromTextAndUrl(
          $text,
          Url::fromUri($item->getUrl()->toString(),
            array(
              'attributes' => array(
                'title' => $item->title,
                'target' => '_blank',
              ),
            )
          )
        )->toString();

        if ($this->getSetting('order') == 'first') {
          $markup = $icon_link . ' ' . $text_link;
        }
        else {
          $markup = $text_link . ' ' . $icon_link;
        }
      }
      else {
        $markup = $icon_link;
      }

      // Return the $markup
      $elements[$delta] = array(
        '#markup' => Markup::create($markup),
      );
    }

    return $elements;
  }
}